==> drawpile-0.9.10.tar.gz: drawpile-0.9.10/src/client/core/rasterop.h <==
/*
   Drawpile - a collaborative drawing program.

   Copyright (C) 2008-2013 Calle Laakkonen

   Drawpile is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Drawpile is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Drawpile.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef RASTEROP_H
#define RASTEROP_H

#include <QString>
#include <array>

namespace paintcore {

struct BlendMode {
	const char *name;      // translatable name
	const QString svgname; // SVG style name of this blending mode
	int id;                // ID as used in the protocol

	bool layermode;        // can this be used as a layer mode?
	bool visible;          // can this mode be shown in mode selection lists?

	BlendMode() : name(nullptr), id(-1) { }
	BlendMode(const char *_name, const QString &_svgname, int _id, bool _layermode=true, bool _visible=true) : name(_name), svgname(_svgname), id(_id), layermode(_layermode), visible(_visible) { }
};

// Note. Protocol ordering and display ordering of the modes differ.
static const int BLEND_MODES=11;
extern const BlendMode BLEND_MODE[BLEND_MODES];

/**
 * Composite a color using a mask onto an image.
 * @param mode composition mode
 * @param base pixels onto which the color is composited
 * @param color ARGB color value
 * @param mask alpha mask
 * @param w width of composition rectangle
 * @param h height of composition rectangle
 * @param maskskip number of bytes to skip to get to the next line in the mask
 * @param baseskip number of (bytes) to skip to get to the next line in the base
 */
void compositeMask(int mode, quint32 *base, quint32 color, const uchar *mask, int w, int h, int maskskip, int baseskip);

/**
 * Composite two equally big image tiles.
 * @param mode composition mode
 * @param base pixels onto which over is composited
 * @parma over the pixels on top of base
 * @param len number of pixels to blend
 * @param opacity blend opacity (0..255)
 */
void compositePixels(int mode, quint32 *base, const quint32 *over, int len, uchar opacity);

/**
 * Get a weighted average of the pixel data using the mask as the weights
 *
 * @param pixels ARGB pixel data
 * @param mask weight mask
 * @param w width of the sampling rectangle
 * @param h height of the sampling rectangle
 * @param maskskip nubmer of bytes to skip to get to the next line in the mask
 * @param pixelskip number of bytes to skip to get to the enxt line in the pixel data
 * @return [weight sum, red, green, blue, alpha]
 */
std::array<quint32, 5> sampleMask(const quint32 *pixels, const uchar *mask, int w, int h, int maskskip, int pixelskip);

/**
 * Add tint to pixel values
 */
void tintPixels(quint32 *pixels, int len, quint32 tint);

/**
 * @brief Get the blending mode for the given SVG composite operation name
 * @return blending mode or -1 if operation is not supported
 */
int blendModeSvg(const QString &name);

/**
 * @brief Get the SVG composition operation name for the given blend mode
 * @param blendmode
 * @return name. The normal blend mode is returned if blendmode is invalid
 */
const QString &svgBlendMode(int blendmode);

}

#endif

==> drawpile-2.1.6.tar.gz: drawpile-2.1.6/src/client/core/rasterop.h <==
/*
   Drawpile - a collaborative drawing program.

   Copyright (C) 2008-2015 Calle Laakkonen

   Drawpile is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Drawpile is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Drawpile.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef PAINTCORE_RASTEROP_H
#define PAINTCORE_RASTEROP_H

#include <Qt>
#include <array>

#include "blendmodes.h"

namespace paintcore {

/**
 * Composite a color using a mask onto an image.
 * @param mode composition mode
 * @param base pixels onto which the color is composited
 * @param color ARGB color value
 * @param mask alpha mask
 * @param w width of composition rectangle
 * @param h height of composition rectangle
 * @param maskskip number of bytes to skip to get to the next line in the mask
 * @param baseskip number of (bytes) to skip to get to the next line in the base
 */
void compositeMask(BlendMode::Mode mode, quint32 *base, quint32 color, const uchar *mask, int w, int h, int maskskip, int baseskip);

/**
 * Composite two equally big image tiles.
 * @param mode composition mode
 * @param base pixels onto which over is composited
 * @parma over the pixels on top of base
 * @param len number of pixels to blend
 * @param opacity blend opacity (0..255)
 */
void compositePixels(BlendMode::Mode mode, quint32 *base, const quint32 *over, int len, uchar opacity);

/**
 * Get a weighted average of the pixel data using the mask as the weights
 *
 * @param pixels ARGB pixel data
 * @param mask weight mask
 * @param w width of the sampling rectangle
 * @param h height of the sampling rectangle
 * @param maskskip nubmer of bytes to skip to get to the next line in the mask
 * @param pixelskip number of bytes to skip to get to the enxt line in the pixel data
 * @return [weight sum, red, green, blue, alpha]
 */
std::array<quint32, 5> sampleMask(const quint32 *pixels, const uchar *mask, int w, int h, int maskskip, int pixelskip);

/**
 * Add tint to pixel values
 */
void tintPixels(quint32 *pixels, int len, quint32 tint);

}

#endif
